@@ -107,9 +107,12 @@ static command_rec ssl_config_cmds[] = {
                "(`XXX:...:XXX' - see manual)")
     AP_SRV_CMD(CertificateFile, TAKE1,
                "SSL Server Certificate file "
-               "(`/path/to/file' - PEM encoded)")
+               "(`/path/to/file' - PEM or DER encoded)")
     AP_SRV_CMD(CertificateKeyFile, TAKE1,
                "SSL Server Private Key file "
+               "(`/path/to/file' - PEM or DER encoded)")
+    AP_SRV_CMD(CertificateChainFile, TAKE1,
+               "SSL Server CA Certificate Chain file "
                "(`/path/to/file' - PEM encoded)")
 #ifdef SSL_EXPERIMENTAL
     AP_ALL_CMD(CACertificatePath, TAKE1,
