@@ -306,6 +306,7 @@
 #define ssl_dbm_delete   dbm_delete
 #define ssl_dbm_firstkey dbm_firstkey
 #define ssl_dbm_nextkey  dbm_nextkey
+#if !defined(SSL_DBM_FILE_SUFFIX_DIR) && !defined(SSL_DBM_FILE_SUFFIX_PAG)
 #if defined(DBM_SUFFIX)
 #define SSL_DBM_FILE_SUFFIX_DIR DBM_SUFFIX
 #define SSL_DBM_FILE_SUFFIX_PAG DBM_SUFFIX
@@ -316,6 +317,7 @@
 #define SSL_DBM_FILE_SUFFIX_DIR ".dir"
 #define SSL_DBM_FILE_SUFFIX_PAG ".pag"
 #endif
+#endif
 #endif /* !SSL_USE_SDBM */
 
 /*
@@ -517,6 +519,7 @@ typedef struct {
     BOOL         bEnabled;
     char        *szPublicCertFile[SSL_AIDX_MAX];
     char        *szPrivateKeyFile[SSL_AIDX_MAX];
+    char        *szCertificateChain;
     char        *szCACertificatePath;
     char        *szCACertificateFile;
     char        *szLogFile;
@@ -585,6 +588,7 @@ const char  *ssl_cmd_SSLEngine(cmd_parms *, char *, int flag);
 const char  *ssl_cmd_SSLCipherSuite(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLCertificateFile(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLCertificateKeyFile(cmd_parms *, char *, char *);
+const char  *ssl_cmd_SSLCertificateChainFile(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLCACertificatePath(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLCACertificateFile(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLCARevocationPath(cmd_parms *, SSLDirConfigRec *, char *);
@@ -603,7 +607,7 @@ const char  *ssl_cmd_SSLRequire(cmd_parms *, SSLDirConfigRec *, char *);
 /*  module initialization  */
 void         ssl_init_Module(server_rec *, pool *);
 void         ssl_init_SSLLibrary(void);
-void         ssl_init_GetCertAndKey(server_rec *, pool *, SSLSrvConfigRec *);
+void         ssl_init_ConfigureServer(server_rec *, pool *, SSLSrvConfigRec *);
 STACK_OF(X509_NAME) 
             *ssl_init_FindCAList(server_rec *, pool *, char *, char *);
 void         ssl_init_Child(server_rec *, pool *);
@@ -636,6 +640,7 @@ void         ssl_callback_LogTracingState(SSL *, int, int);
 
 /*  Session Cache Support  */
 void         ssl_scache_init(server_rec *, pool *);
+void         ssl_scache_kill(void);
 void         ssl_scache_store(server_rec *, SSL_SESSION *, int);
 SSL_SESSION *ssl_scache_retrieve(server_rec *, UCHAR *, int);
 void         ssl_scache_remove(server_rec *, SSL_SESSION *);
@@ -643,12 +648,14 @@ void         ssl_scache_expire(server_rec *, time_t);
 void         ssl_scache_status(server_rec *, pool *, void (*)(char *, void *), void *);
 char        *ssl_scache_id2sz(UCHAR *, int);
 void         ssl_scache_dbm_init(server_rec *, pool *);
+void         ssl_scache_dbm_kill(void);
 void         ssl_scache_dbm_store(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_retrieve(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_remove(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_expire(server_rec *, time_t);
 void         ssl_scache_dbm_status(server_rec *, pool *, void (*)(char *, void *), void *);
 void         ssl_scache_shm_init(server_rec *, pool *);
+void         ssl_scache_shm_kill(void);
 void         ssl_scache_shm_store(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_shm_retrieve(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_shm_remove(server_rec *, ssl_scinfo_t *);
