@@ -188,6 +188,7 @@ void *ssl_config_server_create(pool *p, server_rec *s)
     sc->bEnabled               = UNSET;
     sc->szCACertificatePath    = NULL;
     sc->szCACertificateFile    = NULL;
+    sc->szCertificateChain     = NULL;
     sc->szLogFile              = NULL;
     sc->szCipherSuite          = NULL;
     sc->nLogLevel              = SSL_LOG_NONE;
@@ -231,6 +232,7 @@ void *ssl_config_server_merge(pool *p, void *basev, void *addv)
     cfgMergeBool(bEnabled);
     cfgMergeString(szCACertificatePath);
     cfgMergeString(szCACertificateFile);
+    cfgMergeString(szCertificateChain);
     cfgMergeString(szLogFile);
     cfgMergeString(szCipherSuite);
     cfgMerge(nLogLevel, SSL_LOG_NONE);
@@ -523,6 +525,20 @@ const char *ssl_cmd_SSLCertificateKeyFile(
     return NULL;
 }
 
+const char *ssl_cmd_SSLCertificateChainFile(
+    cmd_parms *cmd, char *struct_ptr, char *arg)
+{
+    SSLSrvConfigRec *sc = mySrvConfig(cmd->server);
+    char *cpPath;
+
+    cpPath = ap_server_root_relative(cmd->pool, arg);
+    if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISREG|SSL_PCM_ISNONZERO, cpPath))
+        return ap_pstrcat(cmd->pool, "SSLCertificateChainFile: file '",
+                          cpPath, "' not exists or empty", NULL);
+    sc->szCertificateChain = cpPath;
+    return NULL;
+}
+
 const char *ssl_cmd_SSLCACertificatePath(
     cmd_parms *cmd, SSLDirConfigRec *dc, char *arg)
 {
