@@ -204,53 +204,6 @@ void ssl_hook_NewConnection(conn_rec *conn)
         BIO_set_callback_arg(SSL_get_rbio(ssl), ssl);
     }
 
-    /*
-     * Configure the server certificate and private key which should be used
-     * for this connection. Under combined RSA+DSA situation there can be two
-     * certs for a server. The SSL library automatically handles this
-     * internally.
-     */
-    if (sc->pPublicCert[SSL_AIDX_RSA] != NULL) {
-        if (SSL_use_certificate(ssl, sc->pPublicCert[SSL_AIDX_RSA]) <= 0) {
-            ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
-                    "Unable to configure RSA server certificate for connection");
-            SSL_free(ssl);
-            ap_ctx_set(fb->ctx, "ssl", NULL);
-            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
-            conn->aborted = 1;
-            return;
-        }
-        if (SSL_use_PrivateKey(ssl, sc->pPrivateKey[SSL_AIDX_RSA]) <= 0) {
-            ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
-                    "Unable to configure server RSA private key for connection");
-            SSL_free(ssl);
-            ap_ctx_set(fb->ctx, "ssl", NULL);
-            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
-            conn->aborted = 1;
-            return;
-        }
-    }
-    if (sc->pPublicCert[SSL_AIDX_DSA] != NULL) {
-        if (SSL_use_certificate(ssl, sc->pPublicCert[SSL_AIDX_DSA]) <= 0) {
-            ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
-                    "Unable to configure DSA server certificate for connection");
-            SSL_free(ssl);
-            ap_ctx_set(fb->ctx, "ssl", NULL);
-            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
-            conn->aborted = 1;
-            return;
-        }
-        if (SSL_use_PrivateKey(ssl, sc->pPrivateKey[SSL_AIDX_DSA]) <= 0) {
-            ssl_log(conn->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
-                    "Unable to configure server DSA private key for connection");
-            SSL_free(ssl);
-            ap_ctx_set(fb->ctx, "ssl", NULL);
-            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
-            conn->aborted = 1;
-            return;
-        }
-    }
-
     /*
      * Predefine some client verification results
      */
@@ -346,8 +299,14 @@ void ssl_hook_NewConnection(conn_rec *conn)
             else if (SSL_get_error(ssl, rc) == SSL_ERROR_SYSCALL) {
                 if (errno == EINTR)
                     continue;
-                ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_SSLERR|SSL_ADD_ERRNO,
-                        "SSL handshake interrupted by system");
+                if (errno > 0)
+                    ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_SSLERR|SSL_ADD_ERRNO,
+                            "SSL handshake interrupted by system "
+                            "[Hint: Stop button pressed in browser?!]");
+                else
+                    ssl_log(srvr, SSL_LOG_INFO|SSL_ADD_SSLERR|SSL_ADD_ERRNO,
+                            "Spurious SSL handshake interrupt"
+                            "[Hint: Usually just one of those OpenSSL confusions!?]");
                 SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
                 SSL_smart_shutdown(ssl);
                 SSL_free(ssl);
@@ -1557,8 +1516,8 @@ int ssl_callback_SSLVerify_CRL(
             BIO_free(bio);
             cp2 = X509_NAME_oneline(subject, NULL, 0);
             ssl_log(s, SSL_LOG_TRACE, "CA CRL: Issuer: %s, %s", cp2, cp);
-            free(cp);
             free(cp2);
+            free(cp);
         }
 
         /*
