@@ -96,6 +96,22 @@ void ssl_scache_init(server_rec *s, pool *p)
     return;
 }
 
+void ssl_scache_kill(void)
+{
+    SSLModConfigRec *mc = myModConfig();
+
+    if (mc->nSessionCacheMode == SSL_SCMODE_DBM)
+        ssl_scache_dbm_kill();
+    else if (mc->nSessionCacheMode == SSL_SCMODE_SHM)
+        ssl_scache_shm_kill();
+
+#ifdef SSL_VENDOR
+    ap_hook_use("ap::mod_ssl::vendor::scache_kill",
+                AP_HOOK_SIG1(void), AP_HOOK_ALL);
+#endif
+    return;
+}
+
 void ssl_scache_store(server_rec *s, SSL_SESSION *pSession, int timeout)
 {
     SSLModConfigRec *mc = myModConfig();
@@ -301,18 +317,27 @@ void ssl_scache_dbm_init(server_rec *s, pool *p)
 
 #ifndef WIN32
     /*
-     * we have to make sure the Apache child processes
-     * have access to the DBM file...
+     * We have to make sure the Apache child processes
+     * have access to the DBM file. But because there
+     * are brain-dead platforms where we cannot exactly
+     * determine the suffixes we try all possibilities.
      */
     if (geteuid() == 0 /* is superuser */) {
-        chown(mc->szSessionCacheDataFile,
-              ap_user_id, -1 /* no gid change */);
-        chown(ap_pstrcat(p, mc->szSessionCacheDataFile,
-                         SSL_DBM_FILE_SUFFIX_DIR, NULL),
-              ap_user_id, -1 /* no gid change */);
-        chown(ap_pstrcat(p, mc->szSessionCacheDataFile,
-                         SSL_DBM_FILE_SUFFIX_PAG, NULL),
-              ap_user_id, -1 /* no gid change */);
+        chown(mc->szSessionCacheDataFile, ap_user_id, -1 /* no gid change */);
+        if (chown(ap_pstrcat(p, mc->szSessionCacheDataFile, SSL_DBM_FILE_SUFFIX_DIR, NULL),
+                  ap_user_id, -1) == -1) {
+            if (chown(ap_pstrcat(p, mc->szSessionCacheDataFile, ".db", NULL),
+                      ap_user_id, -1) == -1)
+                chown(ap_pstrcat(p, mc->szSessionCacheDataFile, ".dir", NULL),
+                      ap_user_id, -1);
+        }
+        if (chown(ap_pstrcat(p, mc->szSessionCacheDataFile, SSL_DBM_FILE_SUFFIX_PAG, NULL),
+                  ap_user_id, -1) == -1) {
+            if (chown(ap_pstrcat(p, mc->szSessionCacheDataFile, ".db", NULL),
+                      ap_user_id, -1) == -1)
+                chown(ap_pstrcat(p, mc->szSessionCacheDataFile, ".pag", NULL),
+                      ap_user_id, -1);
+        }
     }
 #endif
     ssl_mutex_off(s);
@@ -320,6 +345,12 @@ void ssl_scache_dbm_init(server_rec *s, pool *p)
     return;
 }
 
+void ssl_scache_dbm_kill(void)
+{
+    /* nothing to be done */
+    return;
+}
+
 void ssl_scache_dbm_store(server_rec *s, ssl_scinfo_t *SCI)
 {
     SSLModConfigRec *mc = myModConfig();
@@ -346,6 +377,7 @@ void ssl_scache_dbm_store(server_rec *s, ssl_scinfo_t *SCI)
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
                 "Cannot open SSLSessionCache DBM file `%s' for writing (store)",
                 mc->szSessionCacheDataFile);
+        free(dbmval.dptr);
         ssl_mutex_off(s);
         return;
     }
@@ -442,9 +474,9 @@ void ssl_scache_dbm_expire(server_rec *s, time_t tNow)
     datum dbmval;
     time_t tExpiresAt;
     int nElements = 0;
+    int nDeleted = 0;
     int bDelete;
 
-    ssl_log(s, SSL_LOG_TRACE, "Expiring DBM-based SSL session cache");
     ssl_mutex_on(s);
     if ((dbm = ssl_dbm_open(mc->szSessionCacheDataFile,
                             O_RDWR, SSL_DBM_FILE_MODE)) == NULL) {
@@ -468,13 +500,16 @@ void ssl_scache_dbm_expire(server_rec *s, time_t tNow)
             }
             dbmkeythis = dbmkey;
             dbmkey = ssl_dbm_nextkey(dbm);
-            if (bDelete)
+            if (bDelete) {
                 ssl_dbm_delete(dbm, dbmkeythis);
+                nDeleted++;
+            }
         } while (dbmkey.dptr != NULL);
     }
     ssl_dbm_close(dbm);
     ssl_mutex_off(s);
-    ssl_log(s, SSL_LOG_INFO, "Session Cache Status: %d elements", nElements);
+    ssl_log(s, SSL_LOG_TRACE, "Expired DBM-based SSL session cache: "
+            "old: %d, new: %d, removed: %d", nElements+nDeleted, nElements, nDeleted);
     return;
 }
 
@@ -615,7 +650,15 @@ void ssl_scache_shm_init(server_rec *s, pool *p)
      */
     ssl_log(s, SSL_LOG_INFO, 
             "Init: Created hash-table (%d buckets) "
-            "in shared memory (%d bytes)", n, avail);
+            "in shared memory (%d bytes) for SSL session cache", n, avail);
+    return;
+}
+
+void ssl_scache_shm_kill(void)
+{
+    SSLModConfigRec *mc = myModConfig();
+
+    ap_mm_destroy(mc->pSessionCacheDataMM);
     return;
 }
 
@@ -726,7 +769,7 @@ void ssl_scache_shm_expire(server_rec *s, time_t tNow)
         } while (rc == TABLE_ERROR_NONE);
     }
     ssl_mutex_off(s);
-    ssl_log(s, SSL_LOG_INFO, "Expired SHM-based SSL session cache: "
+    ssl_log(s, SSL_LOG_TRACE, "Expired SHM-based SSL session cache: "
             "old: %d, new: %d, removed: %d", nElements+nDeleted, nElements, nDeleted);
     return;
 }
